(* $Id: termdef.mli,v 1.9 2008/11/21 10:31:18 deraugla Exp $ *)

open Rtdecl;
open Util;
open Xlib;
open Keysym;

type term_event =
  [ TermEvButtonPress of int and int and int and modified int
  | TermEvButtonRelease of int and int and int and modified int
  | TermEvButtonMotion of int and int and int
  | TermEvKeyPress of modified keysym
  | TermEvAnswer of string
  | TermEvExtendHistory
  | TermEvSizeChange of int and int ]
;

type term_event_handler = widget -> term_event -> unit;

value cland : char -> int -> int;
value clor : char -> int -> int;
value clxor : char -> int -> int;

value f_bld : int;
value f_ita : int;
value f_blk : int;
value f_und : int;
value f_rev : int;
value f_foreg : int;
value f_backg : int;

type frozen 'a = [ Ffun of unit -> 'a | Fval of 'a ];
type mlazy 'a = ref (frozen 'a);

value freeze : (unit -> 'a) -> mlazy 'a;
value unfreeze : mlazy 'a -> 'a;

value flg_auto_wrap : int;
value flg_newline_mode : int;
value flg_insert_mode : int;
value flg_appl_num_kp : int;
value flg_appl_curs_key : int;
value flg_orig_mode : int;
value flg_cursor_off : int;
value flg_reverse_video : int;
value flg_blink : int;
value flg_blink_string : int;
value flg_timeout_set : int;
value flg_bars : int;
value flg_trace : int;

type input_state =
  [ IS_normal
  | IS_escape
  | IS_ext of int and list int
  | IS_csi of int and list int
  | IS_csi_question of int and list int
  | IS_escape_sharp
  | IS_escape_lparen
  | IS_escape_rparen ]
;

module Gstring :
  sig
    type t = 'abstract;
    type gchar = string;
    value empty : t;
    value length : t -> int;
    value get : t -> int -> gchar;
    value set : t -> int -> gchar -> unit;
    value create : int -> t;
    value make : int -> gchar -> t;
    value sub : t -> int -> int -> t;
    value fill : t -> int -> int -> gchar -> unit;
    value blit : t -> int -> t -> int -> int -> unit;
    value concat : t -> t -> t;
    value of_char : gchar -> t;
    value to_string : t -> string;
  end
;

type term_global_info =
  { dfont : font;
    tgc : gC;
    c_backg : mutable int;
    c_foreg : mutable int;
    c_font : mutable xfont }
and term_local_info =
  { term_gi : term_global_info;
    att_val : attribute_values;
    callb : term_event_handler;
    tfs : array (mlazy font);
    twidth : int;
    theight : int;
    tascent : int;
    tdescent : int;
    max_history_size : mutable int;
    lines : mutable array line;
    nhrow : mutable int;
    nrow : mutable int;
    ncol : mutable int;
    shift : mutable int;
    sreg1 : mutable int;
    sreg2 : mutable int;
    crow : mutable int;
    ccol : mutable int;
    vmask : mutable char;
    foregm : mutable int;
    backgm : mutable int;
    vcrow : mutable int;
    vccol : mutable int;
    vvmask : mutable char;
    vforegm : mutable int;
    vbackgm : mutable int;
    erow1 : mutable int;
    ecol1 : mutable int;
    erow2 : mutable int;
    ecol2 : mutable int;
    scroll_cnt : mutable int;
    scroll_rect : mutable list (int * int * int * int);
    state : mutable input_state;
    tabs : mutable string;
    flags : mutable int }
and line =
  { str : Gstring.t; vid : string; foreg : array int; backg : array int }
and copy_area =
  { src_row : int;
    src_col : int;
    dst_row : int;
    dst_col : int;
    cp_nrow : int;
    cp_ncol : int }
;

value flg_set : term_local_info -> int -> bool;
value flg_reset : term_local_info -> int -> bool;
value set_flg : term_local_info -> int -> unit;
value reset_flg : term_local_info -> int -> unit;
value invert_flg : term_local_info -> int -> unit;

value get_term_global_info : (globinfo -> string) -> term_global_info;
value term_global_info : term_global_info -> globinfo -> string;
value get_term_local_info : (locinfo -> string) -> term_local_info;
value term_local_info : term_local_info -> locinfo -> string;

value term_font : array string;
value term_inter : ref int;
value term_band : ref int;
value term_blink : ref (int * int);

value term_expose_row :
  widget -> term_local_info -> int -> int -> int -> unit;
value term_expose_cursor : widget -> term_local_info -> unit;
value term_expose_blink : widget -> term_local_info -> unit;
value row_col_of_xy : term_local_info -> int -> int -> (int * int);
value term_expose : widget -> int -> int -> int -> int -> unit;
value term_resize : widget -> term_local_info -> int -> int -> unit;
value term_soft_reset : term_local_info -> unit;
value term_scroll_up :
  widget -> term_local_info -> int -> int -> int -> unit;
value term_scroll_down :
  widget -> term_local_info -> int -> int -> int -> unit;
value show_cursor : widget -> term_local_info -> unit;
value hide_cursor : widget -> term_local_info -> unit;

value term_emphasize_from : widget -> int -> int -> unit;
value term_emphasize_to : widget -> int -> int -> unit;
value term_emphasized_location : widget -> ((int * int) * (int * int));
value term_get_emphasized : widget -> string;
